#include <ggi/mapping.h>
#define MAXMP 100

static struct Map
{ 
	struct mm_struct *mm;
	unsigned long	 mapstart,maplen;
} map[MAXMP],*mp;

static int mapcnt=0;

static inline void
unmap_all(void)
{
  for(;mapcnt;mapcnt--)
   {
     mp = &map[mapcnt-1];
     fast_unmap_page_range(mp->mm,mp->mapstart,mp->maplen);
     flush_tlb_range(mp->mm,mp->mapstart,mp->mapstart+mp->maplen);
   }
}

void kgi_unmap_all(void)
{
  unmap_all();
}

void kgi_savescreen(void *whereto)
{
  unsigned char i;
  kgi_unmap_all();
  if (verify_area (VERIFY_WRITE, (void *) whereto, chipinfo.ramsize)) return;
  if (graph_buffer)
    {	
      memcpy_tofs(whereto,(void *)fb_graphics,fb_graph_len);
    } else {
      for (i=0;i<(chipinfo.ramsize>>16);i++)
	{
	  SET_PAGE(i);
	  memcpy_tofs(whereto+(i<<16),(void *)0xA0000,0x10000);
	};
      SET_PAGE(0);
    };
}

void kgi_loadscreen(void *wherefrom)
{ 
  unsigned char i;
  kgi_unmap_all();
  if (verify_area (VERIFY_READ, (void *) wherefrom, chipinfo.ramsize)) return;
  if (graph_buffer)
    {	
      memcpy_fromfs((void *)fb_graphics,wherefrom,fb_graph_len);
    } else {
      for (i=0;i<(chipinfo.ramsize>>16);i++)
	{
	  SET_PAGE(i);
	  memcpy_fromfs((void *)0xA0000,wherefrom+(i<<16),0x10000);
	};
	SET_PAGE(0);
    };
}

void kgi_linearmap_close(struct vm_area_struct * area)
{	
	/* FIXME ... delete only those mappings that belong */
	/* to the closing task - not too critical, but ... */
	kgi_unmap_all();
}

unsigned long kgi_linearmap_nopage(struct vm_area_struct * area, unsigned long address,
	int no_share)
{
  struct kgi_state *gr = (struct kgi_state *) area->vm_offset;

	if (gr->graph_num!=fg_console) {	
		/* A backgrounded process accesses graphics mem ? ! */
		schedule();return (0);
        }

	if (((gr->mode.request.graphtype==GT_TEXT)&&
	     (address -area->vm_start >= 0x8000))
	    || (address -area->vm_start >= chipinfo.ramsize))  
	{	/* Accessing graphics mem beyond limit ! */
		current->blocked &= ~(1<<(SIGSEGV-1));
		send_sig(SIGSEGV, current, 1);
		return (0);
        }

	kgi_unmap_all();

	if (gr->mode.request.graphtype==GT_TEXT)
	{	map[mapcnt].mm=area->vm_mm;
		map[mapcnt].mapstart=area->vm_start;
		map[mapcnt].maplen=0x8000;
		mapcnt++;
		fast_remap_page_range(area->vm_mm,area->vm_start, 
					0xb8000, 0x8000, area->vm_page_prot);
		flush_tlb_range(area->vm_mm,area->vm_start,area->vm_start+0x8000);
	}
	else if (graph_buffer)
	{	map[mapcnt].mm=area->vm_mm;
		map[mapcnt].mapstart=area->vm_start;
		map[mapcnt].maplen=fb_graph_len;
		mapcnt++;
        	fast_remap_page_range(area->vm_mm,area->vm_start,graph_buffer, 
        				fb_graph_len, area->vm_page_prot);
		flush_tlb_range(area->vm_mm,area->vm_start,area->vm_start+fb_graph_len);
        } else {
	  SET_PAGE((address-(area->vm_start))>>16);
	  map[mapcnt].mm=area->vm_mm;
	  map[mapcnt].mapstart=area->vm_start+curr_page*0x10000;
	  map[mapcnt].maplen=0x10000;
	  mapcnt++;
	  fast_remap_page_range(area->vm_mm,area->vm_start+curr_page*0x10000, 
				0xa0000, 0x10000, area->vm_page_prot);
	  flush_tlb_range(area->vm_mm,area->vm_start,area->vm_start+chipinfo.ramsize);
	}
	    
	/* invalidate ok ??? */

        ++current->min_flt;		/* do some statistics */
        
	return((unsigned long)NULL);
}

void kgi_pagedmap_close(struct vm_area_struct * area)
{	
	/* FIXME ... delete only those mappings that belong */
	/* to the closing task - not too critical, but ... */
	kgi_unmap_all();
}


unsigned long kgi_pagedmap_nopage(struct vm_area_struct * area, unsigned long address,
	int no_share)
{
  struct kgi_state *gr = (struct kgi_state *) area->vm_offset;

	if (gr->graph_num!=fg_console) {	
		/* A backgrounded process accesses graphics mem ? ! */
		schedule();return (0);
        }
	if ((gr->mode.request.graphtype==GT_TEXT&&
	     (address -area->vm_start >= 0x8000))
	     ||(address -area->vm_start >= 64*1024)) 
	{	/* Accessing graphics mem beyond 64K page limit ! */
		current->blocked &= ~(1<<(SIGSEGV-1));
		send_sig(SIGSEGV, current, 1);
		return (0);
        }

	kgi_unmap_all();

	if (gr->mode.request.graphtype==GT_TEXT)
	{	map[mapcnt].mm=area->vm_mm;
		map[mapcnt].mapstart=area->vm_start;
		map[mapcnt].maplen=0x8000;
		mapcnt++;
		fast_remap_page_range(area->vm_mm,area->vm_start, 
					0xb8000, 0x8000, area->vm_page_prot);
		flush_tlb_range(area->vm_mm,area->vm_start,area->vm_start+0x8000);
	} else if (graph_buffer){	
	  map[mapcnt].mm=area->vm_mm;
	  map[mapcnt].mapstart=area->vm_start;
	  map[mapcnt].maplen=fb_graph_len;
	  mapcnt++;
	  fast_remap_page_range(area->vm_mm,area->vm_start,
				(unsigned long)graph_buffer+0x10000*sel_page, 
				0x10000, area->vm_page_prot);
	  flush_tlb_range(area->vm_mm,area->vm_start,area->vm_start+0x10000);
        } else {
	  map[mapcnt].mm=area->vm_mm;
	  map[mapcnt].mapstart=area->vm_start;
	  map[mapcnt].maplen=0x10000;
	  mapcnt++;
	  fast_remap_page_range(area->vm_mm,area->vm_start, 
				0xa0000, 0x10000, area->vm_page_prot);
	  flush_tlb_range(area->vm_mm,area->vm_start,area->vm_start+0x10000);
	  SET_PAGE(sel_page);	
	}

/* FIXME ??? Is invalidate() handled with flush_tlb_range ? */
        ++current->min_flt;		/* do some statistics */
        
	return((unsigned long)NULL);
}
