/*\
|*| ---------------------------------------------------------------------------
|*|	AV9194-07/11 clocks (compatible with ICS2494-07/11 ???)
|*| ---------------------------------------------------------------------------
|*|
|*|	Copyright (c) 1995 by Steffen Seeger, [seeger@physik.tu-chemnitz.de]
|*|
|*|	This program is free software; you can redistribute it and/or modify
|*|	it under the terms of the GNU General Public License as published by
|*|	the Free Software Foundation; version 2, or any later version.
|*|
|*|	You should have received a copy of the GNU General Public License
|*|	along with this program; see the file COPYING.  If not, write to
|*|	the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
|*| ---------------------------------------------------------------------------
|*|	The 07 and 11 mask revisions of the AV9194 all provide the same
|*|	DCLK frequencies but have different MCLK values. However, we don't
|*|	care about MCLK (yet).
|*|	Derived from the AV9164/AV90C64 docs by ICS
\*/

#include <linux/kernel.h>

#include <ggi/graphdev.h>
#include <ggi/kgi.h>

static struct kgi_clock clock =
{
	"Avasem",		/* manufacturer		*/
	"AV9194-07/11 "__FILE__,/* model		*/
	GM_ALL,			/* supported modes	*/
	0,			/* special capabilities	*/
	{ 0x7FFF, 0x7FFF },	/* maximum resolution	*/


	CLK_TYPE_FIXED,		/* fixed clock values	*/

	/*\
	|*|	The clock values. Note that they have to be in descending
	|*|	order and that there must be no repetitions.
	\*/

	{{
		130000000,	120000000,	110000000,	94500000,
		 80000000,	 77000000,	 75000000,	65000000,
		 50000000,	 44900000,	 40000000,	36000000,
		 31500000,	 28322000,	 25175000,	0	
	}}
};

/*\
|*|	Driving CS[0:3] to index[i] selects clock.clock.freq[i].
\*/

static unsigned char index[MAX_FIXED_CLOCKS] =
{	0x08, 0x09, 0x0C, 0x0F,
	0x0A, 0x05, 0x0E, 0x0D,
	0x04, 0x07, 0x02, 0x06,
	0x0B, 0x01, 0x00, 0xFF
};

static inline int
clock_init (void)
{
  /* no detection possible */
  return EOK;
}

static inline void
clock_done(void)
{
}

#include "fixed.inc"
