/*****************************************************************************
 *
 *	S3 Trio64 integrated RAMDAC 
 * --------------------------------------------------------------------------
 *
 *	Copyright (C) 1996, Jon Taylor <taylorj@gaia.ecs.csus.edu>
 *
 *	This program is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; either version 2, or (at your option)
 *	any later version.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; see the file COPYING.  If not, write to
 *	the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ****************************************************************************/
#define	MAINTAINER	"Jon Taylor <taylorj@gaia.ecs.csus.edu>"

#ifdef DEBUG
  #define	NEED_GRAPHTYPE_SYMS
#endif

#include "graphdev.h"
#include "kgi.h"
#include "S3/trio64.h"
#define	DAC_PRECISION	6	/* we can do 6 bit R,G,B resolution	*/
#include "IBM/vga_dac.h"
#include "IBM/clut.inc"

#define DAC_VERSION "0.01 ALPHA"

#if !((MONITORVERSION == 01) && (MODEVERSION == 03) && (RAMDACVERSION == 01))
  #warning kgi datastructures have changed, check this code !!
#endif

#define	SUPPORTED (GM_TEXT | GM_1BIT | GM_4BIT | GM_8BIT \
		  | GM_15BIT | GM_16BIT | GM_32BIT)
#define	CLUT_USED (GM_TEXT | GM_1BIT | GM_4BIT | GM_8BIT)

static struct kgi_ramdac ramdac = {
  
  "S3 Incorporated",		/* manufacturer		*/
  "Trio64 integrated DAC",	/* model		*/
  SUPPORTED,			/* supported modes	*/
  0,				/* flags		*/
  { 0x7FFF, 0x7FFF },		/* maximum resolution	*/
  DAC_8BIT_PB | DAC_16BIT_PB,	/* device type		*/
  {        0,  80000000 },	/* normal mode		*/
  { 67500000, 135000000 },	/* 2xclk mode		*/

  {
      {    0, 135000000 },	/* 	GT_TEXT		*/
      {	   0,         0 },	/* 	GT_1BIT		*/
      {	   0, 135000000 },	/* 	GT_4BIT		*/
      {	   0, 135000000 },	/* 	GT_8BIT		*/	
      {	   0, 135000000 },	/* 	GT_15BIT	*/
      {    0, 135000000 },	/* 	GT_16BIT	*/
      {	   0,  73000000 },	/* 	GT_24BIT	*/
      {	   0,  50000000 }	/* 	GT_32BIT	*/
 }
};



/* GGI interface */

int kgi_ramdac_init(void) {
  TRACE(printk("  Trio64 integrated RAMDAC\n"));
  return (EOK);
}

void kgi_ramdac_done(void) {
  TRACE(printk("%s %s driver removed.\n", ramdac.manufact, ramdac.model));
}

struct kgi_ramdac *kgi_ramdac_get_info (void) {
  return &ramdac; 
}

int kgi_ramdac_check_mode (struct kgi_mode *mode, enum kgi_tm_cmd cmd) {
  
  int mul = mode->lclk.mul, div = mode->lclk.div;
  int newclock = mode->dclk;
  
/*  printk ("mul=%d, div=%d\n",mul,div); */

  switch (mode->request.graphtype) { /* First try a setup usign PA[15:0] */
     case GT_TEXT:	mul = 1; div = 1; break;
     case GT_1BIT:
     case GT_4BIT:
     case GT_8BIT:	mul = 1; div = 2; break;
     case GT_15BIT:
     case GT_16BIT:	mul = 1; div = 1; break;
     case GT_24BIT:	mul = 3; div = 1; break;
     case GT_32BIT:	mul = 2; div = 1; break;

     default:		return -ERAMDAC;
   }
  
  switch (cmd) {
  
     case CMD_PROPOSE:
	mode->lclk.mul = mul; mode->clk.mul = 1;
	mode->lclk.div = div; mode->clk.div = 1;
	TRACE(printk("%s:%i: proposing %i:%i multiplexing mode\n", \
	      __FILE__,__LINE__, mul, div));
	return EOK;

     case CMD_LOWER:
	if (   (newclock < ramdac.clock[mode->request.graphtype].min)
	    || (newclock < ramdac.normal.min))
	  return -ERAMDAC;
	
	if (ramdac.clock[mode->request.graphtype].max < newclock)
	  newclock = ramdac.clock[mode->request.graphtype].max;

	if (ramdac.normal.max < newclock*mul/div)
	  newclock = ramdac.normal.max*div/mul;

	mode->dclk = newclock;
	TRACE(printk("%s:LOWER: %i:%i multiplex mode, DCLK = %i Hz\n", \
	      __FILE__, mul, div, mode->dclk));
	return EOK;

     case CMD_RAISE:
	if (   (ramdac.clock[mode->request.graphtype].max < newclock)
	    || (ramdac.normal.max < newclock))
	  return -ERAMDAC;

	if (newclock < ramdac.clock[mode->request.graphtype].min)
	  newclock = ramdac.clock[mode->request.graphtype].min;

	mode->dclk = newclock;
	TRACE(printk("%s:RAISE: %i:%i multiplex mode, DCLK = %i Hz\n", \
	      __FILE__, mul, div, mode->dclk));
	return EOK;

     case CMD_CHECK:
/* !!! implement checking! */
	#warning mode_check not implemented yet!
	return EOK;

    default:
	INTERNALERROR;
	return -ERAMDAC;
  }
}

void kgi_ramdac_set_mode(struct kgi_mode *mode) {
   
  TRACE(printk("%s: graphtype = %s\n", __FILE__, 
	       GRAPHMASK(mode->request.graphtype)));

  if ((1 << mode->request.graphtype) & CLUT_USED)
    set_CLUT_entries(mode->palette);
}

int kgi_ramdac_ioctl (struct inode *inode, struct file *file, unsigned int cmd,
		  unsigned long arg) {
  /* this may set
       -  dac_params
       -  CLUT (or gamma correction table).
   */

  if (get_CLUT_entries == NULL)	/* !!! trick gcc	*/
    return -ERAMDAC;
    
  return -ERAMDAC;
}
