/* 
   Event library for GGI.

   Copyright (C) 1995 Andreas Beck - becka@hp.rz.uni-duesseldorf.de

   If you do any modifications, I would like you to send diffs to me
   to allow for collecting a more and more complete set of event drivers and
   to improve existing ones.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#define _LIB_COMPILE_

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>
#include <signal.h>
#include <sys/fcntl.h>
#include <sys/ioctl.h>
#include <sys/mman.h>
#include <sys/stat.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <sys/time.h>
#include <linux/vt.h>
#include <linux/kd.h>
#include <string.h>

int tochild[2],frchild[2];
int chldpid;

enum EV_type { EV_NONE, EV_COMMAND, EV_KEYBOARD, EV_POINTER 
	     };	/* What more ? */

enum CMD_subtype { CMD_QUIT, CMD_LISTEN_HANDLE, CMD_LISTEN_FILE,
                   CMD_UNLISTEN_HANDLE, CMD_UNLISTEN_FILE,
                   CMD_STOP_REPORT, CMD_START_REPORT } ;

enum KBD_subtype { KBD_PRESS,KBD_RELEASE };

typedef struct	{ char ascii[10];	/* ASCII representation of char */
		  int keynum;		/* Keycode - we should ENUM this ... */
		  int speed,depth;	/* dynamics info for music keyboards */
		} kbddata;

typedef struct
	{ enum EV_type type;
	  int subtype;
	  int major,minor;
	  char data[0];  /* This follows here ... */
	} event;

struct action { int fd;
		void (*do_what)(int fd);
		void (*at_close)(int fd);
		};

int actionanz=0;
struct action actions[20];		

event *get_event(int handle)
{ static char buffer[1024];
  int length;
  read(handle,&length,sizeof(length));
  if (length>1024) return NULL;	/* should do better ;-) */
  read(handle,buffer,length);
  return (event *) buffer;
}

int send_event(int handle,event *ev,void *data,int datalen)
{ int x,len;
  len=sizeof(event)+datalen;
  if ((x=write(handle,&len,sizeof(len)))!=sizeof(len)) return x;
  if ((x=write(handle,ev,sizeof(event)))!=sizeof(event)) return x;
  if ((x=write(handle,data,datalen))!=datalen) return x;
  return 0;
}

int send_command(enum CMD_subtype cmd,void *data,int datalen)
{ event bla = { EV_COMMAND,0,-1,-1 };
  bla.subtype=cmd;
  return send_event(tochild[1],&bla,data,datalen);
}

void kbd_report(int fd)
{ char buf;
  read(fd,&buf,1);printf("KB read %c.\n",buf);
  return;
}

void add_handle(int handle)
{ struct stat stb;

  actions[actionanz].fd=handle;
  fstat(handle,&stb);
  fprintf(stderr,"attaching ...\n");
  
  if (S_ISCHR(stb.st_mode))
  { 
    fprintf(stderr,"char special ...\n");
    switch(stb.st_rdev&0xff00)
    { case 0x0500:	/* cuax ... probably mice */
    	fprintf(stderr,"mouse attached.\n");
    	break;
      case 0x0400:	/* ttys ... hopefully keyboard */
        fprintf(stderr,"keyboard attached.\n");
	actions[actionanz].do_what =kbd_report;
	actions[actionanz].at_close=NULL;
	actionanz++;
        break;
      default:		/* ttys ... hopefully keyboard */
        fprintf(stderr,"unknown device %x.\n",stb.st_rdev);
        break;
    }
  }
  else return;
}

void do_child_cmd(int handle)
{  event *ev;
    ev=get_event(handle);
    switch(ev->type)
    { case EV_COMMAND: 
        switch(ev->subtype)
        { case CMD_QUIT: 
            return; /* Parent wants us to quit ... */
          case CMD_LISTEN_HANDLE:
            add_handle(*(int *)ev->data);
            break;
          case CMD_LISTEN_FILE:
            { int hand;
              if ((hand=open((char *)ev->data,O_RDONLY))!=-1) add_handle(hand);
            }
            break;
          default:
            fprintf(stderr,"Unknown child command.\n");
            break;
        } break;
      default:fprintf(stderr,"Non-command on child-pipe !\n");break;
    }
}

void event_loop_child(void)
{ fd_set readfd;
  int x;
  
  while(1)
  { FD_ZERO(&readfd);
    for(x=0;x<actionanz;x++) FD_SET(actions[x].fd,&readfd);
    
    printf("Select is: %d\n",select(100,&readfd,NULL,NULL,NULL));
    for(x=0;x<actionanz;x++) 
      if (FD_ISSET(actions[x].fd,&readfd)) 
        actions[x].do_what(actions[x].fd);

  }

}


/****************************/
/* initialize event package */
/****************************/
int event_init(void)
{  
  if (pipe(tochild)) return(-1);
  if (pipe(frchild)) return(-1);

  actions[actionanz].fd=tochild[0];
  actions[actionanz].do_what=do_child_cmd;
  actions[actionanz].at_close=NULL;
  actionanz++;

  switch(chldpid=fork())
  { default: return 0;	/* Fork OK - Parent */
    case -1: return -1; /* Fork failed */
    case 0 : event_loop_child();exit(0);	/* Child handles events then exits */
  }

  return -1;	/* Not reached */
}

/******************************/
/* deinitialize event package */
/******************************/
void event_exit(void)
{ send_command(CMD_QUIT,NULL,0);
  close(tochild[0]);close(tochild[1]);
  close(frchild[0]);close(frchild[1]);
  kill(chldpid,SIGHUP);
}

int main(void)
{ event_init();
  send_command(CMD_LISTEN_FILE,"/dev/tty1",10);
  sleep(1);
  send_command(CMD_LISTEN_HANDLE,"\0\0\0\0",4);	/* Add stdin */
  sleep(1);
  event_exit();
  return EXIT_SUCCESS;
}
